package jtaDiscRack.presentation.dpanels;

import jtaDiscRack.presentation.*;
import jtaDiscRack.presentation.delements.*;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;

import org.enhydra.dods.exceptions.AssertionDataObjectException;
/**
 * Creates a table panel.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class DTablePanel extends DPanel {

   public DTablePanel (DCollection myOwner,String title,boolean hasBorder) {

      super(myOwner,2,title,true,hasBorder);

      // creating a table which do not allow cell editing
      Vector columnNames=new Vector();

      Collection c=myOwner.getElementStructure();
      Iterator it=c.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         columnNames.add(el.toName());
      }

      JTable allItems=new JTable(new Vector(),columnNames) {
         public boolean isCellEditable(int row, int col) {
            return false;
         }

         public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
         }

      };

      // setting the first column (ID column) to be invisible
      TableColumnModel tcm=allItems.getColumnModel();
      TableColumn column=allItems.getColumnModel().getColumn(0);
      column.setMinWidth(0);
      column.setMaxWidth(0);
      column.setPreferredWidth(0);
      column.setResizable(false);

      // setting some table properties
      allItems.setColumnSelectionAllowed(false);
      allItems.setRowSelectionAllowed(true);
      allItems.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      allItems.getTableHeader().setReorderingAllowed(false);

      // fills Table
      DefaultTableModel dtm=(DefaultTableModel)allItems.getModel();
      it=myOwner.getTableElements().iterator();
      while (it.hasNext()) {
         DCollectionElement elem=(DCollectionElement)it.next();
         Vector v=new Vector(elem.toComplexTypeValues());
         dtm.addRow(v);
      }

      // mouse listener for editing on double-click
      allItems.addMouseListener(new MouseAdapter() {
         public void mouseClicked (MouseEvent me) {
            if (me.getClickCount()>1) {
               editElementDialog();
            }
         }
      });

      allItems.getInputMap(JComponent.WHEN_FOCUSED).
            put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0,false),"edit");
      allItems.getActionMap().put("edit",new AbstractAction() {
         public void actionPerformed(ActionEvent e) {
            editElementDialog();
         }
      });

      allItems.getInputMap(JComponent.WHEN_FOCUSED).
            put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0,false),"delete");
      allItems.getActionMap().put("delete",new AbstractAction() {
         public void actionPerformed(ActionEvent e) {
            DCollectionElement deleteElement=getSelectedElement();
            int yn=JOptionPane.showConfirmDialog(getWindow(),
                  "Do you really want to delete selected item?",
                  deleteElement.toName()+" Deleting",
                  JOptionPane.YES_NO_OPTION);
            if (yn==JOptionPane.YES_OPTION) {

               try {
                  ((DCollection)getOwner()).onElementDeleted(deleteElement);
                  removeElement(deleteElement);
               } catch (AssertionDataObjectException ex) {
                  JOptionPane.showMessageDialog(getWindow(),
                        "Read-only cache: delete not allowed.","Disc error",
                        JOptionPane.ERROR_MESSAGE);
               } catch (Exception ex) {
                  JOptionPane.showMessageDialog(getWindow(),
                        "Something went wrong, please try again !","Disc error",
                        JOptionPane.ERROR_MESSAGE);
               }
            }
         }
      });


      // creates panel
      JScrollPane allItemsPane=new JScrollPane();
      allItemsPane.setViewportView(allItems);
      Dimension tDim;
      tDim=new Dimension(tableDimension);

      allItems.setPreferredScrollableViewportSize(tDim);

      add(allItemsPane);
      add(Box.createVerticalGlue());
   }

   public JTable getTable () {
      JScrollPane jsp=(JScrollPane)getComponent(0);
      JViewport jvp=(JViewport)jsp.getComponent(0);
      JTable jt=(JTable)jvp.getComponent(0);
      return jt;
   }

   public DCollectionElement getSelectedElement () {
      JTable t=getTable();
      int row=t.getSelectedRow();
      if (row>=0) {
         String ID=(String)t.getValueAt(row, 0);
         Iterator it=((DCollection)getOwner()).getTableElements().iterator();
         while (it.hasNext()) {
            DCollectionElement e=(DCollectionElement)it.next();
            if (e.getID().equals(ID)) {
               return e;
            }
         }
      }
      return null;

   }

   public void modifyElement(DCollectionElement e) {
      JTable t=getTable();
      int row=t.getSelectedRow();
      DefaultTableModel dtm=(DefaultTableModel)t.getModel();
      Vector v=new Vector(((DCollectionElement)e).toComplexTypeValues());
      dtm.removeRow(row);
      dtm.insertRow(row,v);
      try {
         t.setRowSelectionInterval(row,row);
      } catch (Exception ex) {}
   }

   public void removeElement(DCollectionElement e) {
      ((DCollection)getOwner()).remove(getSelectedElement());
      JTable t=getTable();
      int row=t.getSelectedRow();
      DefaultTableModel dtm=(DefaultTableModel)t.getModel();
      dtm.removeRow(row);
      try {
         if (row==0) {
            row++;
         }
         t.setRowSelectionInterval(row-1,row-1);
      } catch (Exception ex) {}
      t.requestFocus();
   }

   private void editElementDialog () {
      DCollectionElement editElement=getSelectedElement();
      if (editElement != null) {
         Window w=getWindow();
         DElementDialog de;
         if (w instanceof JFrame) {
            de=new DElementDialog((JFrame)w,editElement.toName()+" Editing");
         } else {
            de=new DElementDialog((JDialog)w,editElement.toName()+" Editing");
         }
         de.editDElement(editElement.getPanel(),true);
         if (!de.isCanceled()) {
            modifyElement(editElement);
            ((DCollection)getOwner()).onElementModified(editElement);
         }
      } else {
         JOptionPane.showMessageDialog(getWindow(),
               "Please select valid element!",
               "JtaDiscRack",
               JOptionPane.WARNING_MESSAGE);
      }
      getTable().requestFocus();
   }

}
