package discRack.presentation.delements;

import discRack.presentation.dpanels.*;

import java.util.*;

/**
 * Represents a complex element - element made of simple elements and other
 * complex elements and gives a panel for editing it's context.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class DComplexElement extends DSimpleElement {

   protected java.util.List complexStructure=new ArrayList();

   public DComplexElement (String name) {
      super(name);
   }

   /**
   * Returns the collection of elements this element is made of.
   */
   public Collection toComplexType() {
      return complexStructure;
   }

   /**
   * Returns the collection of strings that represents elements
   * that this element is made of.
   */
   public Collection toComplexTypeValues () {
      java.util.List l=new ArrayList();
      Iterator it=complexStructure.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         l.add(el.toValue());
      }
      return l;
   }

   /**
   * Sets the element and all element it is made of to be
   * read only or not.
   */
   public void setReadOnly (boolean ro) {
      super.setReadOnly(ro);
      Iterator it=complexStructure.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         el.setReadOnly(ro);
      }
   }

   public boolean isEmpty () {
      boolean isEmpty=true;
      Iterator it=complexStructure.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         isEmpty=isEmpty && el.isEmpty();
      }
      return isEmpty;
   }

   public boolean isValid () {
      boolean isValid=true;
      Iterator it=complexStructure.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         isValid=isValid && el.isValid();
      }
      return isValid;
   }

   public DPanel getPanel () {
      DSimpleElement[] c=new DSimpleElement[complexStructure.size()];
      complexStructure.toArray(c);
      return new DGroupPanel(this,c,toName(),true,true);
   }

   /** Gets the element that is placed at specified no. within structure. */
   public DSimpleElement get (int no) {
      try {
         return (DSimpleElement)complexStructure.get(no);
      } catch (Exception ex) {
         return null;
      }
   }

   /**
   * Sets the element that is placed at specified no. within structure
   * to the specified value.
   */
   public void set (int no,Object value) {
      DSimpleElement el;
      try {
         el=get(no);
      } catch (Exception ex) {
         el=null;
      }
      if (el != null) {
         el.setValue(value);
      }
   }


   /** Gets the element with specified name from stucture. */
   public DSimpleElement get (String name) {
      Iterator it=complexStructure.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         if (el.name.equals(name)) {
            return el;
         }
      }
      return null;
   }

   /**
   * Sets the element with specified name from stucture to
   * the specified value.
   */
   public void set (String name,Object value) {
      DSimpleElement el=get(name);
      if (el != null) {
         el.setValue(value);
      }
   }

   public String toString () {
      return name;
   }

   protected void fillStructure () {
      return;
   }

}